@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import re, tempfile
+import re, tempfile, sys, subprocess
 
 import owlready2
 from owlready2.base            import *
@@ -80,6 +80,10 @@ _TYPES = { FunctionalProperty, InverseFunctionalProperty, TransitiveProperty, Sy
 
 JAVA_MEMORY = 2000
 
+if (sys.platform == "win32") or (sys.platform == "cygwin"):
+  _subprocess_kargs = { "creationflags" : subprocess.CREATE_NO_WINDOW }
+else:
+  _subprocess_kargs = {}
 
 def _keep_most_specific(s, consider_equivalence = True):
   r = set()
@@ -103,6 +107,13 @@ def _keep_most_specific(s, consider_equivalence = True):
           r.add(i)
   return r
 
+def _decode(s):
+  if not s: return ""
+  try:
+    return s.decode("utf8")
+  except UnicodeDecodeError:
+    return s.decode("latin")
+    
 
 def sync_reasoner_hermit(x = None, infer_property_values = False, debug = 1, keep_tmp_file = False):
   if   isinstance(x, World):    world = x
@@ -134,14 +145,15 @@ def sync_reasoner_hermit(x = None, infer_property_values = False, debug = 1, kee
       t0 = time.time()
       
     try:
-      output = subprocess.check_output(command, stderr = subprocess.STDOUT)
+      output = subprocess.check_output(command, stderr = subprocess.STDOUT, **_subprocess_kargs)
     except subprocess.CalledProcessError as e:
       if (e.returncode == 1) and (b"Inconsistent ontology" in (e.output or b"")):
         raise OwlReadyInconsistentOntologyError()
       else:
-        raise OwlReadyJavaError("Java error message is:\n%s" % (e.stderr or e.output or b"").decode("utf8"))
+        raise OwlReadyJavaError("Java error message is:\n%s" % _decode(e.stderr or e.output or b""))
       
-    output = output.decode("utf8").replace("\r","")
+    output = _decode(output).replace("\r","")
+    
     if debug:
       print("* Owlready2 * HermiT took %s seconds" % (time.time() - t0), file = sys.stderr)
       if debug > 1:
@@ -225,17 +237,19 @@ def sync_reasoner_pellet(x = None, infer_property_values = False, infer_data_pro
     else:                          ontology = world.get_ontology(_INFERRENCES_ONTOLOGY)
     
     tmp = tempfile.NamedTemporaryFile("wb", delete = False)
+    python_name_storid = world._abbreviate("http://www.lesfleursdunormal.fr/static/_downloads/owlready_ontology.owl#python_name")
+    def save_filter(graph, s, p, o, d): return p != python_name_storid
     if isinstance(x, list):
-      for o in x: o.save(tmp, format = "ntriples", commit = False)
+      for o in x: o.save(tmp, format = "ntriples", filter = save_filter, commit = False)
     else:
-      world.save(tmp, format = "ntriples")
+      world.save(tmp, format = "ntriples", filter = save_filter)
     tmp.close()
 
     # Use Jena for loading because OWLAPI is bugged with NTriples.
     command = [owlready2.JAVA_EXE, "-Xmx%sM" % JAVA_MEMORY, "-cp", _PELLET_CLASSPATH, "pellet.Pellet", "realize", "--loader", "Jena", "--input-format", "N-Triples", "--ignore-imports", tmp.name]
     if infer_property_values:      command.insert(-2, "--infer-prop-values")
     if infer_data_property_values: command.insert(-2, "--infer-data-prop-values")
-    
+
     if debug:
       import time
       print("* Owlready2 * Running Pellet...", file = sys.stderr)
@@ -243,18 +257,23 @@ def sync_reasoner_pellet(x = None, infer_property_values = False, infer_data_pro
       t0 = time.time()
     
     try:
-      output = subprocess.run(command, stdout = subprocess.PIPE, stderr = subprocess.PIPE, check = True).stdout
+      output = subprocess.run(command, stdout = subprocess.PIPE, stderr = subprocess.PIPE, check = True, **_subprocess_kargs).stdout
     except subprocess.CalledProcessError as e:
       if (e.returncode == 1) and (b"ERROR: Ontology is inconsistent" in (e.stderr or b"")): # XXX
-        raise OwlReadyInconsistentOntologyError()
-      else:
-        raise OwlReadyJavaError("Java error message is:\n%s" % (e.stderr or e.output or b"").decode("utf8"))
-
-    try:
-      output = output.decode("utf8").replace("\r","")
-    except UnicodeDecodeError:
-      output = output.decode("latin").replace("\r","")
+        msg = _decode(e.stderr or e.output or b"")
+        
+        if debug > 1:
+          command_explain = [owlready2.JAVA_EXE, "-Xmx%sM" % JAVA_MEMORY, "-cp", _PELLET_CLASSPATH, "pellet.Pellet", "explain", "--ignore-imports", tmp.name]
+          process = subprocess.run(command_explain, stdout = subprocess.PIPE, stderr = subprocess.PIPE, check = False, **_subprocess_kargs)
+          msg += "\nThis is the output of `pellet explain`: \n {}\n{}".format(_decode(process.stdout), _decode(process.stderr))
+          
+        raise OwlReadyInconsistentOntologyError("Java error message is: %s" % msg)
       
+      else:
+        raise OwlReadyJavaError("Java error message is:\n%s" % _decode((e.stderr or e.output or b"")))
+
+    output = _decode(output).replace("\r","")
+    
     if debug:
       print("* Owlready2 * Pellet took %s seconds" % (time.time() - t0), file = sys.stderr)
       if debug > 1:
